﻿using System;
using System.Collections.Generic;

namespace Plugins
{
    public class UCCNCplugin //Class name must be UCCNCplugin to work! 
    {

        // ======================================================================================== Plugin framework

        Plugininterface.Entry UC;

        List<ButtonData> ButtonList = new List<ButtonData>();

        //Called when the plugin is initialised.
        //The parameter is the Plugin interface object which contains all functions prototypes for calls and callbacks.
        public void Init_event(Plugininterface.Entry UC)
        {
            this.UC = UC;
            try
            {
                for (int i = 0; i < 10; ++i)
                    ButtonList.Add(new ButtonData(UC, UC.Readkey("SwitchOutput", "Button" + i.ToString(), "")));
            }
            catch (Exception ex)
            {
            }
        }

        //Called when the plugin is loaded, the author of the plugin should set the details of the plugin here.
        public Plugininterface.Entry.Pluginproperties Getproperties_event(Plugininterface.Entry.Pluginproperties Properties)
        {
            string _verstr = "0.0";
            _verstr = System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString(); // Get version from Assembly info
            while (_verstr.Length > 1 && _verstr.Substring(_verstr.Length - 2, 2) == ".0")
                _verstr = _verstr.Substring(0, _verstr.Length - 2);
            if (_verstr.IndexOf(".") < 0) _verstr += ".0";
            Properties.author = "Szentkereszty Csaba";
            Properties.pluginname = "Switch output";
            Properties.pluginversion = _verstr;
            return Properties;
        }

        //This is a direct function call addressed to this plugin dll
        //The function can be called by macros or by another plugin
        //The passed parameter is an object and the return value is also an object
        public object Informplugin_event(object Message)
        {
            if (Message == null) return true;
            return null;
        }

        //Called when the user presses a button on the UCCNC GUI or if a Callbutton function is executed.
        //The int buttonnumber parameter is the ID of the caller button.
        // The bool onscreen parameter is true if the button was pressed on the GUI and is false if the Callbutton function was called.
        public void Buttonpress_event(int ButtonNumber, bool OnScreen)
        {
            try
            {
                foreach (ButtonData chkbutton in ButtonList)
                    if (chkbutton.ButtonNumber == ButtonNumber)
                        chkbutton.Toggle();
            }
            catch (Exception ex)
            {
            }
        }

        // ======================================================================================== class ButtonData

        class ButtonData
        {
            private Plugininterface.Entry UC;
            public int ButtonNumber { get; private set; }
            private int Port;
            private int Pin;
            private bool ActiveLow;
            public bool IsValid { get; private set; }
            private bool State;

            public ButtonData(Plugininterface.Entry uc, string setupstr)
            {
                UC = uc;
                IsValid = true;
                string[] pars = setupstr.Split(',');
                IsValid &= (pars.Length == 4);
                if (IsValid)
                {
                    int btn;
                    IsValid &= Int32.TryParse(pars[0], out btn);
                    IsValid &= Int32.TryParse(pars[1], out Port);
                    IsValid &= Int32.TryParse(pars[2], out Pin);
                    IsValid &= Boolean.TryParse(pars[3], out ActiveLow);
                    ButtonNumber = btn;
                    IsValid &= UC.IsValidPortPin(Port, Pin, true);
                }
            }

            public void Toggle()
            {
                if (!IsValid)
                    return;
                if (State)
                    Off();
                else
                    On();
            }

            private void On()
            {
                if (ActiveLow)
                    UC.Clroutpin(Port, Pin);
                else
                    UC.Setoutpin(Port, Pin);
                State = true;
                UC.Switchbutton(true, true, ButtonNumber);
            }

            private void Off()
            {
                if (ActiveLow)
                    UC.Setoutpin(Port, Pin);
                else
                    UC.Clroutpin(Port, Pin);
                State = false;
                UC.Switchbutton(true, false, ButtonNumber);
            }
        }

        // ========================================================================================

    }
}
